@@ -16,33 +16,38 @@ predhy.GUI <- function(){
                       observed and the predicted trait values.')),
           tabPanel(h2('Input genotype'),title='Genotype Input',
              fluidRow(column(width=4,fileInput('inbred_gen1',label = h4('inbred_gen'))),
-                      column(width = 8,helpText('a matrix for genotypes of parental lines in numeric format, coded as 1, 0 and
+                      column(width = 8,helpText('A matrix for genotypes of parental lines in numeric format, coded as 1, 0 and
                                                 -1. The row.names of inbred_gen must be provied. It can be obtained from the original genotype using',code('convertgen'),'function')))),
-                      tabPanel(h2('Input phenotype'),title = 'Phenotype Input',
-                         fluidRow(column(width = 4,fileInput('hybrid_phe1',label = h4('hybrid_phe'))),
-                                  column(width=8,helpText('a data frame with three columns. The first column and the second column are the
+          tabPanel(h2('Input hybrid phenotype'),title = 'Hybrid Phenotype',
+             fluidRow(column(width = 4,fileInput('hybrid_phe1',label = h4('hybrid_phe'))),
+                      column(width=8,helpText('A data frame with three columns. The first column and the second column are the
                                               names of male and female parents of the corresponding hybrids, respectively;the third column is the phenotypic values of hybrids. The names of male and
                                               female parents must match the rownames of inbred_gen. Missing (NA) valuesare not allowed.')))),
-                      tabPanel(h2('Input design matrix of the fixed effects & domiance genotypes'),title='Optional Input',
-                         fluidRow(column(width=4,radioButtons('cvfixinput',label = h4('design matrix of the fixed effects(Optional)'),
+          tabPanel(h2('Input parent phenotype'),title='Parent Phenotype',
+                   fluidRow(column(width=4,radioButtons('cvinbredpheinput',label = h4('A matrix of a phenotypic values of parent (Optional)'),
+                                                        c('Not included'='NULL','Input parent phenotype'='input'))),
+                            column(width=8,conditionalPanel(condition = 'input.cvinbredpheinput=="input"',
+                                                            fileInput('inbred_phe',label=h4('Parent Phenotype')))))),
+          tabPanel(h2('Input design matrix of the fixed effects & domiance genotypes'),title='Optional Input',
+             fluidRow(column(width=4,radioButtons('cvfixinput',label = h4('design matrix of the fixed effects(Optional)'),
                                          c('Not included'='NULL','Input a design matrix'='input'))),
-                                  column(width=8,conditionalPanel(condition = 'input.cvfixinput=="input"',fileInput('fix',label=h4('fixed effects'))))),
-                         fluidRow(column(width=4,radioButtons('cvgendinput',label=h4('domiance genotypes(Optional)'),
+                      column(width=8,conditionalPanel(condition = 'input.cvfixinput=="input"',fileInput('fix',label=h4('fixed effects'))))),
+             fluidRow(column(width=4,radioButtons('cvgendinput',label=h4('domiance genotypes(Optional)'),
                                          c('Not included'='NULL','Include domiance genotypes'='input'))))),
-                      tabPanel(h2('Select models & other parameters'),title = 'Parameters',
-                         fluidRow(column(width=6,selectInput('cvmethod',label=h4('method,eight GS methods'),
+          tabPanel(h2('Select models & other parameters'),title = 'Parameters',
+             fluidRow(column(width=6,selectInput('cvmethod',label=h4('method,eight GS methods'),
                                          choices = list('GBLUP'='GBLUP','BayesB'='BayesB',
                                                    'RKHS'='RKHS','PLS'='PLS',
                                                    'LASSO'='LASSO','EN'='EN',
-                                                   'XGBOOST'='XGBOOST','RF'='RF','ALL'='ALL'),
+                                                   'XGBoost'='XGBoost','LightGBM'='LightGBM','ALL'='ALL'),
                                                    selected = 'GBLUP'))),
-                         fluidRow(column(width=6,numericInput('nfold',label=h4('the number of folds'), value = 5))),
-                         fluidRow(column(width=4,numericInput('ntimes',label=h4('replicates'), value = 1)),
+             fluidRow(column(width=6,numericInput('nfold',label=h4('the number of folds'), value = 5))),
+             fluidRow(column(width=4,numericInput('ntimes',label=h4('replicates'), value = 1)),
                                   column(width=4,numericInput('cvseed',label=h4('the random number'),value = 133))),
-                         fluidRow(column(width=4,numericInput('cpu',label=h4('the number of CPU'), value = 1)))),
+             fluidRow(column(width=4,numericInput('cpu',label=h4('the number of CPU'), value = 1)))),
                       tabPanel(h2('Trait predictability (R^2)'),title = 'CV Results',
-                         fluidRow(column(width=4,h3(textOutput('cv1')))),
-                         fluidRow(column(width=12,plotOutput('cvp'))))
+             fluidRow(column(width=4,h3(textOutput('cv1')))),
+             fluidRow(column(width=12,plotOutput('cvp'))))
                                     )),
           tabPanel(h4('predhy.predict'),
           navlistPanel(widths = c(3,9),
@@ -50,22 +55,28 @@ predhy.GUI <- function(){
                       helpText('Predict all potential crosses of a given set of parents using a subset of crosses as the training sample.')),
              tabPanel(h2('Input genotype & phenotype'),title = 'Input files',
                 fluidRow(column(width=4,fileInput('inbred_gen',label = h4('inbred_gen'))),
-                         column(width=8,helpText('a matrix for genotypes of parental lines in numeric format, coded as 1, 0 and
+                         column(width=8,helpText('A matrix for genotypes of parental lines in numeric format, coded as 1, 0 and
                                       -1. The row.names of inbred_gen must be provied. It can be obtained from the original genotype using',code('convertgen'),'function'))),
                 fluidRow(column(width = 4,fileInput('hybrid_phe',label = h4('hybrid_phe'))),
-                         column(width=8,helpText('a data frame with three columns. The first column and the second column are the
+                         column(width=8,helpText('A data frame with three columns. The first column and the second column are the
                                       names of male and female parents of the corresponding hybrids, respectively;the third column is the phenotypic values of hybrids. The names of male and
-                                      female parents must match the rownames of inbred_gen. Missing (NA) valuesare not allowed.')))),
+                                      female parents must match the rownames of inbred_gen. Missing (NA) valuesare not allowed.'))),
+                fluidRow(column(width=4,radioButtons('inbredpheinput1',label = h4('A matrix of a phenotypic values of parent (Optional)'),
+                                                     c('Not included'='NULL','Input parent phenotype'='input'))),
+                         column(width=8,conditionalPanel(condition = 'input.inbredpheinput1=="input"',
+                                                         fileInput('inbred_phe1',label=h4('Parent Phenotype')))))),
              tabPanel(h2('Select methods & models'),title = 'Methods & Models',
                 fluidRow(column(width=6,selectInput('method',label=h4('method,eight GS methods'),
                                          choices = list('GBLUP'='GBLUP','BayesB'='BayesB',
                                          'RKHS'='RKHS','PLS'='PLS',
                                          'LASSO'='LASSO','EN'='EN',
-                                         'XGBOOST'='XGBOOST','RF'='RF'),
+                                         'XGBoost'='XGBoost','LightGBM'='LightGBM'),
                                           selected = 'GBLUP'))),
                 fluidRow(column(width = 6,selectInput('model',label=h4('the prediction model'),
                                           choices = list('the additive model'='A',
-                                                         'the additive-dominance model'='AD'),
+                                                         'the additive-dominance model'='AD',
+                                                         'the additive-phenotypic model'='A-P',
+                                                         'the additive-dominance-phenotypic model'='AD-P'),
                                           selected = 'the additive model')))),
              tabPanel(h2('Select hybrids'),title = 'Selection',
                 fluidRow(column(width=6,selectInput('select',label = h4('the selection of hybrids based on the prediction results'),
@@ -74,7 +85,7 @@ predhy.GUI <- function(){
                                         'the bottom n crosses'='bottom')))),
                 fluidRow(column(width=6,numericInput('number',label=h4('the number of selected top or bottom hybrids,only when select = "top" or select = "bottom".'),
                                                      value = 100)))),
-             tabPanel(h2('Phenotypic values of the predicted hybrids.'),title='Phenotypic values',
+             tabPanel(h2('Phenotypic values of the predicted hybrids'),title='Phenotypic values',
                 fluidRow(column(width = 6,downloadLink('predres',label = h4('Predict & Download Results')))),
                 fluidRow(column(width=12,DT::dataTableOutput('predhyres1')))))),
           tabPanel(h4('predhy.predict_NCII'),
@@ -83,12 +94,16 @@ predhy.GUI <- function(){
                       helpText('Predict all potential crosses of a given set of parents using a subset of crosses as the training sample.')),
              tabPanel(h2('Input genotype & phenotype'),title = 'Input files',
                 fluidRow(column(width=4,fileInput('inbred_gen_NCII',label = h4('inbred_gen'))),
-                         column(width=8,helpText('a matrix for genotypes of parental lines in numeric format, coded as 1, 0 and
+                         column(width=8,helpText('A matrix for genotypes of parental lines in numeric format, coded as 1, 0 and
                                       -1. The row.names of inbred_gen must be provied. It can be obtained from the original genotype using',code('convertgen'),'function'))),
                 fluidRow(column(width = 4,fileInput('hybrid_phe_NCII',label = h4('hybrid_phe'))),
-                                column(width=8,helpText('a data frame with three columns. The first column and the second column are the
+                                column(width=8,helpText('A data frame with three columns. The first column and the second column are the
                                       names of male and female parents of the corresponding hybrids, respectively;the third column is the phenotypic values of hybrids. The names of male and
-                                      female parents must match the rownames of inbred_gen. Missing (NA) valuesare not allowed.')))),
+                                      female parents must match the rownames of inbred_gen. Missing (NA) valuesare not allowed.'))),
+                fluidRow(column(width=4,radioButtons('inbredpheinput2',label = h4('A matrix of a phenotypic values of parent (Optional)'),
+                                                     c('Not included'='NULL','Input parent phenotype'='input'))),
+                         column(width=8,conditionalPanel(condition = 'input.inbredpheinput2=="input"',
+                                                         fileInput('inbred_phe2',label=h4('Parent Phenotype')))))),
              tabPanel(h2('Input names of parents'),title = 'Parent names',
                 fluidPage(column(width = 6,fileInput('male_name',label = h4('male_name'))),
                           column(width = 6,helpText('a vector of the names of male parents.'))),
@@ -98,11 +113,13 @@ predhy.GUI <- function(){
                 fluidRow(column(width=6,selectInput('method_NCII',label=h4('method,eight GS methods'),
                                         choices = list('GBLUP'='GBLUP','BayesB'='BayesB',
                                                        'RKHS'='RKHS','PLS'='PLS','LASSO'='LASSO','EN'='EN',
-                                                       'XGBOOST'='XGBOOST','RF'='RF'),
+                                                       'XGBoost'='XGBoost','LightGBM'='LightGBM'),
                                                   selected = 'GBLUP'))),
                 fluidRow(column(width = 6,selectInput('model_NCII',label=h4('the prediction model'),
                                 choices = list('the additive model'='A',
-                                               'the additive-dominance model'='AD'),
+                                               'the additive-dominance model'='AD',
+                                               'the additive-phenotypic model'='A-P',
+                                               'the additive-dominance-phenotypic model'='AD-P'),
                                 selected = 'the additive model')))),
              tabPanel(h2('Select hybrids'),title = 'Selection',
                 fluidRow(column(width=6,selectInput('select_NCII',label = h4('the selection of hybrids based on the prediction results'),
@@ -111,7 +128,7 @@ predhy.GUI <- function(){
                                                        'the bottom n crosses'='bottom')))),
                 fluidRow(column(width=6,numericInput('number_NCII',label=h4('the number of selected top or bottom hybrids,only when select = "top" or select = "bottom".'),
                                                      value = 100)))),
-             tabPanel(h2('Phenotypic values of the predicted hybrids.'),title='Phenotypic values',
+             tabPanel(h2('Phenotypic values of the predicted hybrids'),title='Phenotypic values',
                 fluidRow(column(width = 6,downloadLink('predres_NCII',label = h4('Predict & Download Results')))),
                 fluidRow(column(width=12,DT::dataTableOutput('predhyres_NCII'))))
              )),
@@ -167,8 +184,13 @@ predhy.GUI <- function(){
     
 ###################################server#######################################    
   server <- function(input, output) {
-    options(shiny.maxRequestSize=1024*1024^2)
+    options(shiny.maxRequestSize=10*1024^3) #max upload = 10G
 ####################################cv##########################################    
+  fix <- reactive({
+    req(input$fix)
+    fix <-as.matrix(fread(input$fix$datapath,header = T,stringsAsFactors=F))
+  })
+    
   inbred <- reactive({
     req(input$inbred_gen1)
     inbred <-as.data.frame(fread(input$inbred_gen1$datapath,header = T,stringsAsFactors=F))
@@ -178,11 +200,14 @@ predhy.GUI <- function(){
   
   hybrid <- reactive({
     req(input$hybrid_phe1)
-    inbred <-as.data.frame(fread(input$hybrid_phe1$datapath,header = T,stringsAsFactors=F))
+    hybrid <-as.data.frame(fread(input$hybrid_phe1$datapath,header = T,stringsAsFactors=F))
   })
   
-  y <- reactive({
-    hybrid()[,3]
+  inbred_phe <- reactive({
+    req(input$inbred_phe)
+    inbred_phe <-as.data.frame(fread(input$inbred_phe$datapath,header = T,stringsAsFactors=F))
+    row.names(inbred_phe) <- inbred_phe[,1]
+    inbred_phe <- as.matrix(inbred_phe[,2,drop=FALSE])
   })
   
   infered <- reactive({
@@ -192,55 +217,72 @@ predhy.GUI <- function(){
   gena <- reactive({
     infered()$add
   })
-  
+
   gend <- reactive({
     infered()$dom
   })
-  
+
   cvmethod <- reactive({switch(input$cvmethod,
                                'GBLUP'='GBLUP','BayesB'='BayesB',
                                'RKHS'='RKHS','PLS'='PLS',
                                'LASSO'='LASSO','EN'='EN',
-                               'XGBOOST'='XGBOOST','RF'='RF',
+                               'XGBoost'='XGBoost','LightGBM'='LightGBM',
                                'ALL'='ALL')})
-  
+
   nfold <- reactive({
     input$nfold
   })
-  
+
   ntimes <- reactive({
     input$ntimes
   })
-  
+
   cvseed <- reactive({
     input$cvseed
   })
-  
+
   cpu <- reactive({
     input$cpu
   })
-  
+
   cvres <- reactive({
     if(input$cvfixinput=='input'){
       if(input$cvgendinput=='input'){
-        cv(fix=fix(),gena = gena(),gend = gend(),y=y(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu())
+        if(input$cvinbredpheinput=='input'){
+          cv(fix=fix(),gena = gena(),gend = gend(),parent_phe = inbred_phe(),hybrid_phe =hybrid(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu(),drawplot = F)
+        }else{
+          cv(fix=fix(),gena = gena(),gend = gend(),parent_phe = NULL,hybrid_phe=hybrid(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu(),drawplot = F)
+        }
       }else{
-        cv(fix=fix(),gena = gena(),gend = NULL,y=y(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu())  
+        if(input$cvinbredpheinput=='input'){
+          cv(fix=fix(),gena = gena(),gend = NULL,parent_phe = inbred_phe(),hybrid_phe=hybrid(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu(),drawplot = F)
+        }else{
+          cv(fix=fix(),gena = gena(),gend = NULL,parent_phe = NULL,hybrid_phe=hybrid(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu(),drawplot = F)
+        }
+
       }
     }else{
       if(input$cvgendinput=='input'){
-        cv(fix=NULL,gena = gena(),gend = gend(),y=y(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu())
+        if(input$cvinbredpheinput=='input'){
+          cv(fix=NULL,gena = gena(),gend = gend(),parent_phe = inbred_phe(),hybrid_phe=hybrid(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu(),drawplot = F)
+        }else{
+          cv(fix=NULL,gena = gena(),gend = gend(),parent_phe = NULL,hybrid_phe=hybrid(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu(),drawplot = F)
+        }
       }else{
-        cv(fix=NULL,gena = gena(),gend = NULL,y=y(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu())
+        if(input$cvinbredpheinput=='input'){
+          cv(fix=NULL,gena = gena(),gend = NULL,parent_phe = inbred_phe(),hybrid_phe=hybrid(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu(),drawplot = F)
+        }else{
+          cv(fix=NULL,gena = gena(),gend = NULL,parent_phe = NULL,hybrid_phe=hybrid(),method=cvmethod(),nfold=nfold(),nTimes=ntimes(),seed=cvseed(),CPU=cpu(),drawplot = F)
+        }
       }
     }
   })
-  
-  output$cv1 <- renderText({ 
+
+  output$cv1 <- renderText({
     if(cvmethod()!='ALL'){
       paste(cvmethod(),'R^2=',mean(cvres()))}
   })
-  
+
   output$cvp <-renderPlot({
     if(cvmethod()=='ALL'){
       mycolor <- c("#66C2A5", "#FC8D62", "#8DA0CB", "#E78AC3", "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3")
@@ -262,7 +304,7 @@ predhy.GUI <- function(){
     req(input$inbred_gen)
     fix <-as.data.frame(fread(input$inbred_gen$datapath,header = T,stringsAsFactors=F))
     row.names(fix) <- fix[,1]
-    fix <- fix[,-1]
+    fix <- as.matrix(fix[,-1])
   })
   
   hybrid_phe <-reactive({
@@ -270,16 +312,25 @@ predhy.GUI <- function(){
     fix <-as.data.frame(fread(input$hybrid_phe$datapath,header = T,stringsAsFactors=F))
   })
   
+  inbred_phe1 <- reactive({
+    req(input$inbred_phe1)
+    inbred_phe1 <-as.data.frame(fread(input$inbred_phe1$datapath,header = T,stringsAsFactors=F))
+    row.names(inbred_phe1) <- inbred_phe1[,1]
+    inbred_phe1 <- as.matrix(inbred_phe1[,2,drop=FALSE])
+  })
+  
   method <- reactive({switch(input$method,
                              'GBLUP'='GBLUP','BayesB'='BayesB',
                              'RKHS'='RKHS','PLS'='PLS',
                              'LASSO'='LASSO','EN'='EN',
-                             'XGBOOST'='XGBOOST','RF'='RF')
+                             'XGBoost'='XGBoost','LightGBM'='LightGBM')
     })
   
   model <- reactive({switch(input$model,
                             'A'='A',
-                            'AD'='AD')
+                            'AD'='AD',
+                            'A-P'='A-P',
+                            'AD-P'='AD-P')
     })
   
   select <- reactive({switch(input$select,
@@ -291,7 +342,11 @@ predhy.GUI <- function(){
   number <- reactive({ifelse(select()=='all',NULL,input$number)})
   
   pred <- reactive({
-    predhy.predict(inbred_gen(),hybrid_phe(),method(),model(),select(),number())
+    if(input$inbredpheinput1=='input'){
+      predhy.predict(inbred_gen(),hybrid_phe = hybrid_phe(),parent_phe = inbred_phe1(),method(),model(),select(),number())
+    }else{
+      predhy.predict(inbred_gen(),hybrid_phe = hybrid_phe(),parent_phe = NULL,method(),model(),select(),number())
+    }
   })
   
   output$predhyres1 <- DT::renderDataTable({
@@ -319,6 +374,13 @@ predhy.GUI <- function(){
     fix <-as.data.frame(fread(input$hybrid_phe_NCII$datapath,header = T,stringsAsFactors=F))
     })
   
+  inbred_phe2 <- reactive({
+    req(input$inbred_phe2)
+    inbred_phe2 <-as.data.frame(fread(input$inbred_phe2$datapath,header = T,stringsAsFactors=F))
+    row.names(inbred_phe2) <- inbred_phe2[,1]
+    inbred_phe2 <- as.matrix(inbred_phe2[,2,drop=FALSE])
+  })
+  
   male_name <- reactive({
     req(input$male_name)
     fix <-c(fread(input$male_name$datapath,header = T,stringsAsFactors=F))[[1]]
@@ -333,12 +395,14 @@ predhy.GUI <- function(){
                                   'GBLUP'='GBLUP','BayesB'='BayesB',
                                   'RKHS'='RKHS','PLS'='PLS',
                                   'LASSO'='LASSO','EN'='EN',
-                                  'XGBOOST'='XGBOOST','RF'='RF')
+                                  'XGBoost'='XGBoost','LightGBM'='LightGBM')
     })
   
   model_NCII <- reactive({switch(input$model_NCII,
                                  'A'='A',
-                                 'AD'='AD')
+                                 'AD'='AD',
+                                 'A-P'='A-P',
+                                 'AD-P'='AD-P')
     })
   
   select_NCII <- reactive({switch(input$select_NCII,
@@ -350,10 +414,19 @@ predhy.GUI <- function(){
   number_NCII <- reactive({ifelse(select_NCII() == 'all',NULL,input$number_NCII)})
   
   pred_NCII <- reactive({
+    if(input$inbredpheinput2=='input'){
     predhy.predict_NCII(inbred_gen = inbred_gen_NCII(),hybrid_phe = hybrid_phe_NCII(),
+                        parent_phe = inbred_phe2(),
                         male_name = male_name(),female_name = female_name(),
                         method = method_NCII(),model = model_NCII(),
                         select = select_NCII(),number = number_NCII())
+    }else{
+    predhy.predict_NCII(inbred_gen = inbred_gen_NCII(),hybrid_phe = hybrid_phe_NCII(),
+                        parent_phe = NULL,
+                        male_name = male_name(),female_name = female_name(),
+                        method = method_NCII(),model = model_NCII(),
+                        select = select_NCII(),number = number_NCII())
+    }
   })
   
   output$predhyres_NCII <- DT::renderDataTable({
